@@ -270,7 +270,7 @@ param.slcafit <- function(
    type <- match.arg(type)
 
    if (se) {
-      vcov <- vcov.slcafit(object, type)
+      vcov <- vcov.slcafit(object, type, ...)
       var <- diag(vcov)
       est <- numeric(length(var))
       est[var >= 0] <- sqrt(var[var >= 0])
@@ -281,7 +281,6 @@ param.slcafit <- function(
          logit = object$logit,
       )
    }
-
    skeleton <- object$skeleton$par
 
    res <- utils::relist(est, skeleton)
@@ -330,20 +329,31 @@ print.slcapar <- function(
 
 
 #' @exportS3Method stats::vcov slcafit
-vcov.slcafit <- function(object, type = c("probs", "logit"), ...) {
+vcov.slcafit <- function(
+   object, type = c("probs", "logit"), method = c("score", "hessian"), ...
+) {
    type <- match.arg(type)
+   method <- match.arg(method)
    id <- object$arg$id
-   score <- object$score
-   fi <- crossprod(score)
-   vcov <- matrix(0, nrow(fi), ncol(fi), dimnames = dimnames(fi))
-   nan <- apply(score, 2, anyNA)
-   vcov[!nan, !nan] <- MASS::ginv(fi[!nan, !nan])
+   dm <- length(id)
+   dn <- paste0("(", seq_len(dm), ")")
+   vcov <- matrix(0, dm, dm, dimnames = list(dn, dn))
+
+   if (method == "hessian") {
+      hess <- object$hess
+      nan <- is.na(diag(hess))
+      vcov[!nan, !nan] <- MASS::ginv(hess[!nan, !nan])
+   } else if (method == "score") {
+      score <- object$score
+      fi <- crossprod(score)
+      nan <- apply(score, 2, anyNA)
+      vcov[!nan, !nan] <- MASS::ginv(fi[!nan, !nan])
+   }
 
    if (type == "probs") {
       jac <- bdiag(tapply(object$par, id, jmat))
       vcov <- jac %*% vcov %*% t(jac)
    }
-
    vcov
 }
 
@@ -426,13 +436,13 @@ predict.slcafit <- function(
 #'
 #' @exportS3Method stats::confint slcafit
 confint.slcafit <- function(
-   object, parm, level = 0.95, type = c("param", "logit"), ...
+   object, parm, level = 0.95, type = c("probs", "logit"), ...
 ) {
    if (missing(parm)) parm <- seq_along(object$par)
    type <- match.arg(type)
 
-   logit <- object$logit
-   vcov <- stats::vcov(object, "logit")
+   val <- switch(type, probs = exp(object$par), logit = object$logit)
+   vcov <- vcov.slcafit(object, type, ...)
    vars <- diag(vcov)
    se <- vars
    se[] <- 0
@@ -442,11 +452,8 @@ confint.slcafit <- function(
    upper <- 1 - lower
    cn <- format_pc(c(lower, upper), 3)
 
-   ci_logit <- logit + se %o% stats::qnorm(c(lower, upper))
-   ci_par <- ci_logit
-   ci_par[] <- exp(unlist(apply(ci_logit, 2, tapply, object$arg$id, norm2)))
-
-   ci <- switch(type, param = ci_par[parm,], logit = ci_logit[parm,])
+   ci <- val + se %o% stats::qnorm(c(lower, upper))
+   ci <- ci[parm,]
    colnames(ci) <- cn
    rownames(ci) <- paste0("(", parm, ")")
    ci
@@ -455,6 +462,7 @@ confint.slcafit <- function(
 format_pc <- function(perc, digits)
    paste(format(100 * perc, trim = TRUE, scientific = FALSE, digits = digits), "%")
 
+
 #' Reorder Latent Class Membership of Latent Class Variables
 #'
 #' Reorders the latent class membership for specified latent class variables in an `slcafit` object.
@@ -574,6 +582,5 @@ reorder.slcafit <- function(x, ...) {
    x$posterior <- list(
       marginal = lapply(post, t), joint = joint
    )
-
    x
 }
